@@ -1,5 +1,9 @@
 import shlex, re, os, shutil, logging
 import subprocess as sp
+from collections import abc
+from . import filter_utils
+
+PIPE = sp.PIPE
 
 # list of global options (gathered on 4/9/21)
 global_options = (
@@ -40,7 +44,6 @@ def parse_options(args):
     """
     if isinstance(args, str):
         args = shlex.split(args)
-
     res = {}
     n = len(args)
     i = 0
@@ -48,7 +51,12 @@ def parse_options(args):
         key = args[i][1:]
         i += 1
         if i < n and args[i][0] != "-":
-            res[key] = args[i]
+            if key not in res:
+                res[key] = args[i]
+            elif isinstance(res[key], str):
+                res[key] = [res[key], args[i]]
+            else:
+                res[key].push(args[i])
             i += 1
         else:
             res[key] = None
@@ -91,7 +99,7 @@ def parse(cmdline):
         args = args[1:]
 
     # identify -i options
-    ipos = [i for i in range(len(args)) if args[i] == "-i"]
+    ipos = [i for i, v in enumerate(args) if v == "-i"]
     ninputs = len(ipos)
     inputs = [None] * ninputs
     if ninputs:
@@ -148,9 +156,25 @@ def compose(global_options={}, inputs=[], outputs=[], command="", shell_command=
     def opts2args(opts):
         args = []
         for key, val in opts.items():
-            args.append(f"-{key}")
-            if val is not None:
-                args.append(str(val))
+            if (
+                (key == "vf" or key == "af" or key == "filter_complex")
+                and val is not None
+                and not isinstance(val, str)
+            ):
+                val = (
+                    filter_utils.compose_graph(*val[:-1], **val[-1])
+                    if isinstance(val[-1], dict)
+                    else filter_utils.compose_graph(*val)
+                )
+
+            karg = f"-{key}"
+            if not isinstance(val, str) and isinstance(val, abc.Sequence):
+                for v in val:
+                    args.extend([karg, str(v)])
+            else:
+                args.append(karg)
+                if val is not None:
+                    args.append(str(val))
         return args
 
     def inputs2args(inputs):
@@ -175,7 +199,6 @@ def compose(global_options={}, inputs=[], outputs=[], command="", shell_command=
         *inputs2args(inputs),
         *outputs2args(outputs),
     ]
-
     return shlex.join(args) if shell_command else args
 
 
@@ -286,9 +309,10 @@ def run_sync(
         args.insert(1, "-hide_banner")
 
     ret = sp.run(args, *sp_arg, stdout=stdout, stderr=stderr, **sp_kwargs)
-    if ret.returncode != 0:
-        print(ret.stderr)
-        raise Exception(f"execution failed\n   {shlex.join(args)}\n\n{ret.stderr.decode('utf-8')}")
+    if ret.returncode != 0 and ret.stderr is not None:
+        raise Exception(
+            f"execution failed\n   {shlex.join(args)}\n\n{ret.stderr.decode('utf-8')}"
+        )
     return ret.stdout
 
 
@@ -301,11 +325,7 @@ def run(args, *sp_arg, hide_banner=True, stdout=sp.PIPE, stderr=sp.PIPE, **sp_kw
     if hide_banner:
         args.insert(1, "-hide_banner")
 
-    ret = sp.run(args, *sp_arg, stdout=stdout, stderr=stderr, **sp_kwargs)
-    if ret.returncode != 0:
-        print(ret.stderr)
-        raise Exception(f"execution failed\n   {shlex.join(args)}\n\n{ret.stderr.decode('utf-8')}")
-    return ret.stdout
+    return sp.Popen(args, *sp_arg, stdout=stdout, stderr=stderr, **sp_kwargs)
 
 
 def ffprobe(
